/***************************************************************************
 *   Copyright (C) 2005 by Bernd Juraschek   *
 *   vdrmon@bjusystems.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qpixmap.h>
#include <qbitmap.h>
#include <qpainter.h>
#include <qdatetime.h>

#include "iconloader.h"
#include "iconmagic.h"
#include "settings.h"

/**
 * construction
 * @param iconLoader the iconloader object
 */
iconmagic::iconmagic(iconloader * iconLoader)
{
  this->iconLoader = iconLoader;
}

/**
 * destruction
 */
iconmagic::~iconmagic()
{
}

/**
 * create the icon for the countdown
 * @param countDown the remaining time in seconds
 * @return the created icon
 */
QPixmap iconmagic::createCountDownIcon(int countDown)
{
  int hours = countDown / 3600;
  int minutes = (countDown/60) % 60;
  int seconds = countDown % 60;

  // too large?
  if (hours >= 100)
    return iconLoader->iconLong;

  // create new icon
  QPixmap icon = createEmptyIcon(22, 10);

  // add icons
  if (hours > 0 || minutes > 0)
  {
    mergeIcons(icon, 0, 0, iconLoader->iconDigits[hours/10]);
    mergeIcons(icon, 5, 0, iconLoader->iconDigits[hours%10]);
    mergeIcons(icon, 10, 0, iconLoader->iconColon);
    mergeIcons(icon, 12, 0, iconLoader->iconDigits[minutes/10]);
    mergeIcons(icon, 17, 0, iconLoader->iconDigits[minutes%10]);
  }
  else
  {
    mergeIcons(icon, 3, 0, iconLoader->iconDigits[seconds/10]);
    mergeIcons(icon, 8, 0, iconLoader->iconDigits[seconds%10]);
    mergeIcons(icon, 13, 0, iconLoader->iconSeconds);
  }

  return icon;
}

/**
 * create the icon for the timer
 * @param timer the start time of the next scheduled recording(s)
 * @return the created icon
 */
QPixmap iconmagic::createTimerIcon(time_t timer)
{
  QDateTime startTime;
  startTime.setTime_t(timer);

  // create new icon
  QPixmap icon = createEmptyIcon(22, 10);

  // show time when lesser than one day offset
  if (QDateTime::currentDateTime().addDays(1) > startTime)
  {
    // show time
    int hour = startTime.time().hour();
    int minute = startTime.time().minute();
    mergeIcons(icon, 0, 0, iconLoader->iconDigits[hour/10]);
    mergeIcons(icon, 5, 0, iconLoader->iconDigits[hour%10]);
    mergeIcons(icon, 10, 0, iconLoader->iconColon);
    mergeIcons(icon, 12, 0, iconLoader->iconDigits[minute/10]);
    mergeIcons(icon, 17, 0, iconLoader->iconDigits[minute%10]);
  }
  else
  {
    // show day
    int day = startTime.date().day();
    QValueList<QPixmap> icons;

    icons.append(iconLoader->iconDigits[day/10]);
    icons.append(iconLoader->iconDigits[day%10]);
    icons.append(iconLoader->iconDot);
    insertIcons(icon, icons);
  }

  return icon;
}

/**
 * create the icon showing the current activity
 * @param plays true if vdr replays a recording
 * @param records the number of running recordings
 * @return the created icon
 */
QPixmap iconmagic::createActionIcon(bool plays, int records)
{
  QValueList<QPixmap> icons;

  QPixmap icon = createEmptyIcon(22, 10);

  // play indicator
  if (plays)
    icons.append(iconLoader->iconPlays);

  // one or multiple record indicators if enough space
  // or the count of the running recordings and one
  // indicator
  if ((plays ? 1 : 0) + records <= 3)
  {
    for (int i = 0; i < records; i++)
      icons.append(iconLoader->iconRecords);
  }
  else
  {
    int max = plays ? 9 : 99;
    if (records <= max)
    {
      if (records >= 10)
        icons.append(iconLoader->iconDigits[records/10]);
      icons.append(iconLoader->iconDigits[records%10]);
    }
    else
    {
      icons.append(iconLoader->iconGreater);
      if (max >= 10)
        icons.append(iconLoader->iconDigits[max/10]);
      icons.append(iconLoader->iconDigits[max%10]);
    }
    icons.append(iconLoader->iconRecords);
  }

  insertIcons(icon, icons);

  return icon;
}

/**
 * create the disk space indicator
 * @param space the disk space information
 * @param prefs the user settings
 * @param full true if the disk full indicator should be displayed
 * @return the created icon
 */
QPixmap iconmagic::createSpaceIcon(diskspace& space, settings * prefs, bool full)
{
  // create empty icon
  QPixmap icon = createEmptyIcon(22, 12);

  // bar width and color
  int width = 22 * space.getPercent() / 100;
  QColor color = space.isLow() ? Qt::yellow : Qt::darkGreen;
  if (full && (time(0) % (2 * prefs->toggleTimerInterval)) < prefs->toggleTimerInterval)
    color = Qt::red;

  // draw rectangle into mask
  const QBitmap * mask = icon.mask();
  QPainter painter;
  painter.begin(mask);
  painter.setPen(QPen(Qt::color1, 1));
  painter.drawRect(0, 0, 22, 12);
  painter.fillRect(1, 1, width-1, 12, QBrush(Qt::color1));
  painter.end();

  // draw rectangle
  painter.begin(&icon);
  painter.setPen(QPen(Qt::black, 1));
  painter.drawRect(0, 0, 22, 12);
  painter.fillRect(1, 1, width-1, 10, QBrush(color));
  painter.end();

  return icon;
}

/**
 * merge multiple icons centered into the destination icon
 * @param destIcon the destination icon
 * @param icons the list of icons to merge
 * @return the merged icon
 */
void iconmagic::insertIcons(QPixmap& destIcon, QValueList<QPixmap> icons)
{
  // calculate needed width
  int width = 0;
  for (unsigned i = 0; i < icons.count(); i++)
  {
    if (!icons[i].isNull())
      width += icons[i].width() + 1;
  }

  // no trailing space
  if (width > 0)
    width--;

  // offset
  int offset = (destIcon.width() - width)/2;

  // add icons
  for (unsigned i = 0; i < icons.count(); i++)
  {
    if (!icons[i].isNull())
    {
      mergeIcons(destIcon, offset, 0, icons[i]);
      offset += icons[i].width() + 1;
    }
  }
}

/**
 * create a emtpy transparent icon
 * @param w the wanted icon width
 * @param h the wanted icon heigth
 * @return the created icon
 */
QPixmap iconmagic::createEmptyIcon(int w, int h)
{
  // create icon
  QPixmap icon(w, h);

  // set empty mask
  QBitmap mask(w, h);
  mask.fill(Qt::color0);
  icon.setMask(mask);

  return icon;
}

/**
 * merge an icon into another icon
 * @param iconDest the icon to merge into
 * @param x the x position to merge to
 * @param y the y position to merge to
 * @param iconInsert the icon to merge
 * @return the merged icon
 */
void iconmagic::mergeIcons(QPixmap& iconDest, int x, int y, QPixmap iconInsert)
{
  // merge masks
  const QBitmap * maskDest = iconDest.mask();
  QBitmap maskInsert = *iconInsert.mask();
  QPainter painter;
  painter.begin(maskDest);
  painter.drawPixmap(x, y, maskInsert);
  painter.end();

  // merge icons
  painter.begin(&iconDest);
  painter.drawPixmap(x, y, iconInsert);
  painter.end();
}


