/***************************************************************************
 *   Copyright (C) 2005 by Bernd Juraschek   *
 *   vdrmon@bjusystems.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "icons.h"
#include "settings.h"
#include "diskspace.h"

icons::icons(settings * prefs, const QString& baseDir) : iconLoader(baseDir), iconMagic(&iconLoader)
{
  lastAction = NoAction;
  lastMode = NoMode;
  lastTimerMode = NoTimer;
  lastEngineFrame = -1;
  lastCountDown = -1;
  lastPlays = 0;
  lastRecords = -1;
  lastSpace = -1;
  lastSpaceLowPercent = 0;
  lastSpaceMonitor = false;
  lastFull = false;
  lastToggle = false;
  this->prefs = prefs;
}


icons::~icons()
{
}

QPixmap icons::getIcon(bool connected, bool plays, int records, time_t timer,
                        diskspace& space, int neededSpace, bool& changed)
{
  // get engine icon
  bool engineChanged;
  setEngineIcon(connected, plays || records > 0, engineChanged);

  // get space icon
  bool spaceChanged;
  setSpaceIcon(space, neededSpace > space.getFree(), spaceChanged);

  // get action icon
  bool actionChanged;
  setActionIcon(connected, plays, records, actionChanged);

  // get timer icon, if needed
  Mode mode = ShowAction;
  int wait = timer - time(0);
  if (prefs->showTimer && timer > 0 && wait > 0 && connected &&
          (time(0) % (2 * prefs->toggleTimerInterval) >= prefs->toggleTimerInterval ||
          lastAction == Idle && prefs->dontToggleIfIdle ||
          wait < 60))
      mode = ShowCountDown;

  bool countDownChanged = false;
  if (mode == ShowCountDown && prefs->showTimer)
  {
      setTimerIcon(timer, countDownChanged);
  }

  // mode change
  bool modeChanged = mode != lastMode;
  lastMode = mode;

  // changes?
  if (!engineChanged && !modeChanged && !spaceChanged &&
       (mode == ShowAction && !actionChanged || mode == ShowCountDown && !countDownChanged))
  {
    changed = false;
    return lastIcon;
  }

  // create new icon
  lastIcon = iconMagic.createEmptyIcon(22, 22);
  // insert action or countdown
  if (mode == ShowAction)
    iconMagic.mergeIcons(lastIcon, 0, 0, actionIcon);
  else
    iconMagic.mergeIcons(lastIcon, 0, 0, timerIcon);

  // insert engine or disk space
  if (prefs->spaceMonitor && prefs->showSpace && (prefs->spaceMode == 0 || prefs->spaceMode >= 1 && space.isFull(neededSpace)) ||
                                                  prefs->spaceMode == 2 && space.isLow())
    iconMagic.mergeIcons(lastIcon, 0, 10, spaceIcon);
  else
    iconMagic.mergeIcons(lastIcon, 0, 10, engineIcon);

  changed = true;
  return lastIcon;
}

void icons::setTimerIcon(time_t timer, bool& changed)
{
  if (prefs->timerMode == 0)
  {
    // calculate countdown
    int countDown = timer - time(0);
    if (countDown > 360000)
      countDown = 360000;

    // changed?
    if (countDown == lastCountDown && lastTimerMode == TimerCountDown)
    {
      changed = false;
      return;
    }
    lastCountDown = countDown;
    lastTimerMode = TimerCountDown;

    // get new icon
    timerIcon = iconMagic.createCountDownIcon(countDown);
    changed = true;
  }
  else
  {
    if (lastTimer == timer && lastTimerMode == TimerTimer)
    {
      changed = false;
      return;
    }
    lastTimer = timer;
    lastTimerMode = TimerTimer;

    // get new icon
    timerIcon = iconMagic.createTimerIcon(timer);
    changed = true;
  }
}

void icons::setActionIcon(bool connected, bool plays, int records, bool& changed)
{
  Action action;

  if (!connected)
  {
    action = Disconnected;
    actionIcon = iconLoader.iconUnknown;
  }
  else if (!records && !plays)
  {
    action = Idle;
    actionIcon = iconLoader.iconIdle;
  }
  else
  {
    action = Active;
    actionIcon = iconMagic.createActionIcon(plays, records);
  }

  if (action != lastAction || plays != lastPlays || records != lastRecords)
  {
    lastAction = action;
    lastPlays = plays;
    lastRecords = records;
    changed = true;
  }
  else
    changed = false;
}

void icons::setEngineIcon(bool connected, bool active, bool& changed)
{
  // get next icon if active
  if (connected && active || lastEngineFrame < 0)
  {
    lastEngineFrame = ++lastEngineFrame % (sizeof(iconLoader.iconEngine) /
                                  sizeof(iconLoader.iconEngine[0]));
    engineIcon = iconLoader.iconEngine[lastEngineFrame];
    changed = true;
  }
  else
    changed = false;
}

void icons::setSpaceIcon(diskspace& space, bool full, bool& changed)
{
  bool toggle = (time(0) % (2 * prefs->toggleTimerInterval)) < prefs->toggleTimerInterval;

  if (lastSpace != space.getPercent() || lastSpaceLowPercent != prefs->spaceLowPercentFull ||
      lastSpaceMonitor != prefs->spaceMonitor || lastFull != full || (lastFull == full && lastToggle != toggle))
  {
    lastSpace = space.getPercent();
    lastSpaceLowPercent = prefs->spaceLowPercentFull;
    lastSpaceMonitor = prefs->spaceMonitor;
    lastFull = full;
    lastToggle = toggle;
    spaceIcon = iconMagic.createSpaceIcon(space, prefs, full);
    changed = true;
  }
  else
    changed = false;
}
