/***************************************************************************
 *   Copyright (C) 2005 by Bernd Juraschek   *
 *   vdrmon@bjusystems.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <klocale.h>
#include <kaction.h>
#include <qtimer.h>
#include <qsocket.h>
#include <kpopupmenu.h>
#include <kconfigdialogmanager.h>
#include <kpassivepopup.h>
#include <kmessagebox.h>
#include <kapplication.h>
#include <qdatetime.h>
#include <sys/vfs.h>

#include "status.h"
#include "kvdrmon.h"
#include "config.h"
#include "settings.h"
#include "audioplayer.h"
#include "tooltip.h"
#include "shutdown.h"

kvdrmon::kvdrmon(status * parent, const QString& baseDir, KApplication * app)
  : KSystemTray(parent), trayIcons(&prefs, baseDir), diskSpace(&prefs), toolTip(this)
{
  // save parent
  dlgStatus = parent;
  this->app = app;

  // create actions for menu items
  KAction * propertiesMenu = new KAction(i18n("&Properties"), KShortcut::null(), this, SLOT(showConfig()), actionCollection(), "showProperties");
  shutdownMenu = new KToggleAction(i18n("&Shutdown after (next) recording"), KShortcut::null(), this, "shutdown");
  connect(shutdownMenu, SIGNAL(toggled(bool)), this, SLOT(toggleShutdown(bool)));

  // create menu
  menu = contextMenu();

  // add actions
  propertiesMenu->plug(menu);
  shutdownMenu->plug(menu);

  // quit uses a signal
  connect(this, SIGNAL(quitSelected()), this, SLOT(quitKvdrmon()));


  // some inits
  active = false;
  plays = false;
  records = 0;
  connected = false;
  nextStart = 0;
  vdrSocket = 0;
  lastDiskFull = false;
  lastDiskLow = false;
  autoShutdown = false;
  shutdownDialog = NULL;

  // read config
  loadConfig();

  // timer
  timerConnect = new QTimer(this);
  timerBlink = new QTimer(this);
  timerAlive = new QTimer(this);
  timerTimeout = new QTimer(this);
  timerSpace = new QTimer(this);

  connect(timerConnect, SIGNAL(timeout()), this, SLOT(connectRetry()));
  connect(timerBlink, SIGNAL(timeout()), this, SLOT(animateIcon()));
  connect(timerTimeout, SIGNAL(timeout()), this, SLOT(disconnectVdr()));
  connect(timerAlive, SIGNAL(timeout()), this, SLOT(sendAlive()));
  connect(timerSpace, SIGNAL(timeout()), this, SLOT(queryStatus()));
  connect(timerSpace, SIGNAL(timeout()), this, SLOT(updateToolTip()));

  timerConnect->start(500);
  timerBlink->start(200);
}

kvdrmon::~kvdrmon()
{
}

void kvdrmon::loadConfig()
{
  configDialog = new preferences(this);
  connect(configDialog, SIGNAL(okClicked()), this, SLOT(saveConfig()));
  connect(configDialog, SIGNAL(applyClicked()), this, SLOT(saveConfig()));

  prefs.readConfig();

  configManager = new KConfigDialogManager(configDialog, &prefs);
  configManager->updateWidgets();
}

void kvdrmon::saveConfig()
{
  // save old host and port
  QString host = prefs.vdrHost;
  int port = prefs.vdrPort;

  configManager->updateSettings();
  prefs.writeConfig();

  // reconnect if host or port has changed
  if (prefs.vdrHost != host || prefs.vdrPort != port)
  {
    disconnectVdr();
  }
  else
  {
    startAlive();
  }
}

void kvdrmon::showConfig()
{
  configDialog->exec();
}

void kvdrmon::quitKvdrmon()
{
  exit(0);
}

void kvdrmon::connectRetry()
{
  if (vdrSocket != 0 && (vdrSocket->state() == QSocket::HostLookup || vdrSocket->state() == QSocket::Connecting))
    return;

  connectVdr();
}

void kvdrmon::connectVdr()
{
  // already connected?
  if (connected)
    return;

  dlgStatus->setStatus(i18n("connecting ..."));

  // create socket
  if (vdrSocket == 0)
    vdrSocket = new QSocket(this);

  // set signalisation
  connect(vdrSocket, SIGNAL(connected()),            this, SLOT(vdrConnected()));
  connect(vdrSocket, SIGNAL(connectionClosed()),     this, SLOT(vdrDisconnected()));
  connect(vdrSocket, SIGNAL(readyRead()),            this, SLOT(vdrReadyRead()));
  connect(vdrSocket, SIGNAL(delayedCloseFinished()), this, SLOT(vdrDisconnected()));

  // connect
  vdrSocket->connectToHost(prefs.vdrHost, prefs.vdrPort);
}

void kvdrmon::disconnectVdr()
{
  stopAlive();
  vdrSocket->close();
  vdrDisconnected();
}

void kvdrmon::vdrConnected()
{
  // change status
  connected = true;
  timerConnect->stop();
  dlgStatus->setStatus(i18n("connected"));
  startAlive();
  timerSpace->start(30000);
  toolTip.setStatus(true);
}

void kvdrmon::vdrDisconnected()
{
  stopAlive();
  connected = false;
  plays = false;
  records = 0;
  vdrSocket->close();
  vdrSocket->deleteLater();
  vdrSocket = 0;
  timerSpace->stop();
  timerConnect->start(500);
  dlgStatus->setStatus(i18n("disconnected"));
  dlgStatus->clearPlaying();
  running.clear();
  showPopup();
  dlgStatus->fillRecordings(running);
  toolTip.setStatus(false);
}

void kvdrmon::vdrReadyRead()
{
  // check for complete line
  while (vdrSocket->canReadLine())
  {
    handleEvent(vdrSocket->readLine());
  }
}

void kvdrmon::handleEvent(const QString& msg)
{
  // parse line
  QString type = msg.left(4).upper();
  QString arg = msg.mid(4).stripWhiteSpace();

  QString neededVersion("0.6");

  if (type == "VERS")
  {
    if (arg != neededVersion)
    {
      KMessageBox::sorry(this,
                         i18n("This version of kvdrmon needs this version of the vdrmon VDR plugin") +
                             ": " + neededVersion, i18n("Bad VDR plugin version"));
      exit(0);
    }
  }
  else if (type == "NACC")
  {
    // access denied
  }
  else if (type == "CREC")
  {
    // clear recording status
    running.clear();
  }
  else if (type == "AREC")
  {
    // active recording
    running.add(arg);
  }
  else if (type == "NREC")
  {
    // active recordings count
    int newrecordings = arg.toInt();
    if (newrecordings > records)
      playStartSound();
    else if (newrecordings < records)
      playStopSound();
    records = newrecordings;

    running.finish();
    showPopup();
    dlgStatus->fillRecordings(running);
    toolTip.setRecording(running);
  }
  else if (type == "CNXT")
  {
    // clear next recordings
    scheduled.clear();
    nextStart = 0;
  }
  else if (type == "NEXT")
  {
    // next scheduled recording changed
    scheduled.add(arg);
    nextStart = arg.left(8).toULong(0, 16);
  }
  else if (type == "NNXT")
  {
    // no next recording
    scheduled.finish();
    toolTip.setScheduled(scheduled);
    toolTip.setStart(nextStart);
  }
  else if (type == "NPLY")
  {
    // no more playing
    plays = false;
    dlgStatus->clearPlaying();
    toolTip.setPlaying(QString(""));
  }
  else if (type == "PLAY")
  {
    // playing active
    plays = true;
    dlgStatus->setPlaying(arg);
    toolTip.setPlaying(arg);
  }
  else if (type == "ALIV")
  {
    // alive response
    // refresh timeout timer
    timerTimeout->start(prefs.aliveTimeout);
  }
  else if (type == "DISK")
  {
    // disk space
    checkSpace(arg);
    toolTip.setSpace(diskSpace);
  }
  else if (type == "DOWN")
  {
    // time for auto shutdown
    QString text = i18n("&Shutdown after (next) recording");
    if (arg == "0")
    {
      stoptime = 0;
      shutdownMenu->setText(text);

    }
    else
    {
      stoptime = arg.left(8).toULong(0, 16);
      QDateTime time;
      time.setTime_t(stoptime);
      shutdownMenu->setText(text + " (" + time.toString("yyyy-MM-dd, hh:mm") + ")");
    }
    updateToolTip();
  }
  else if (type == "SHUT")
  {
    // auto shutdown
    doShutdown();
  }

  updateToolTip();
}

void kvdrmon::animateIcon()
{
  bool changed;
  QPixmap icon = trayIcons.getIcon(connected, plays, records, nextStart, diskSpace, getNeededSpace(), changed);
  if (changed)
    setPixmap(icon);
}

int kvdrmon::getNeededSpace()
{
  int neededSpace = running.getNeededSpace(prefs.mbPerHour);
  neededSpace += scheduled.getNeededSpace(prefs.mbPerHour);
  return neededSpace;
}


void kvdrmon::playStartSound()
{
  if (prefs.startSoundActive)
    audioplayer::play(prefs.startSoundFile);
}

void kvdrmon::playStopSound()
{
  if (prefs.stopSoundActive)
    audioplayer::play(prefs.stopSoundFile);
}

void kvdrmon::showPopup()
{
  if (prefs.showEventPopup)
  {
    QString text = running.getPopupText();
    if (text.length() > 0)
      KPassivePopup::message(running.getPopupText(), this);
  }
}

void kvdrmon::sendCommand(QString cmd)
{
  vdrSocket->writeBlock(cmd.latin1(), cmd.length());
}

void kvdrmon::sendAlive()
{
  sendCommand("ALIV\n");
}

void kvdrmon::startAlive()
{
  stopAlive();
  if (prefs.sendAlive)
  {
    timerAlive->start(prefs.aliveInterval * 1000);
    timerTimeout->start(prefs.aliveTimeout * 1000);
  }
}

void kvdrmon::stopAlive()
{
  timerAlive->stop();
  timerTimeout->stop();
}

void kvdrmon::closeEvent(QCloseEvent * e)
{
  QString text = "";

  // show warning if requested
  if (prefs.warnRecording && records > 0)
    text = i18n("You have requestet to shutdown, but VDR is recording.");
  else if (prefs.warnScheduled)
  {
    int offset = nextStart - time(0);
    int warnTime = prefs.warnScheduledTimeValue;
    switch (prefs.warnScheduledTimeUnit)
    {
      case 0:
        // seconds
        break;
      case 1:
        // minutes
        warnTime *= 60;
        break;
      case 2:
        // hours
        warnTime *= 3600;
        break;
      case 3:
        // days
        warnTime *= 3600 * 24;
        break;
    }

    // time to warn?
    if (offset <= warnTime)
    {
      text = i18n("You have requestet to shutdown, but recordings are scheduled.");
      text += i18n("Next recording starts at:");
      QDateTime next;
      next.setTime_t(nextStart);
      text += " " + next.toString(Qt::LocalDate);
    }
  }

  if (!text.isEmpty())
  {
    int rc = KMessageBox::questionYesNo(this,
              i18n("You have requestet to shutdown, but VDR is recording.\n\nDo you want to continue with shutdown?"),
              i18n("kvdrmon - warning"), KStdGuiItem::yes(), KStdGuiItem::no(), QString::null,
              KMessageBox::Dangerous);

    if (rc != KMessageBox::Yes)
      e->ignore();
  }
}

void kvdrmon::queryStatus()
{
  // query status
//  sendCommand("GETR\nGETN\n");

  // query disk space
  if (prefs.spaceMonitor)
    sendCommand("DISK\n");
}

void kvdrmon::checkSpace(QString space)
{
  // active?
  if (!prefs.spaceMonitor)
    return;

  QStringList values = QStringList::split(' ', space);
  diskSpace = diskspace(&prefs, values[0].toULong(0, 10), values[1].toULong(0, 10), values[2].toULong(0, 10));

  bool played = false;
  bool full = diskSpace.isFull(getNeededSpace());
  if (full != lastDiskFull)
  {
    lastDiskFull = full;
    if (full)
    {
      if (prefs.noSpaceSoundActive)
      {
        audioplayer::play(prefs.noSpaceSoundFile);
        played = true;
      }
      if (prefs.showEventPopup)
      {
        KPassivePopup::message(i18n("Not enough free diskspace for recordings!"), this);
      }
    }
  }

  bool low = diskSpace.isLow();
  if (low != lastDiskLow)
  {
    lastDiskLow = low;
    if (low)
    {
      if (!played && prefs.lowSpaceSoundActive)
        audioplayer::play(prefs.lowSpaceSoundFile);
      if (prefs.showEventPopup)
      {
        KPassivePopup::message(i18n("Diskspace low!"), this);
      }
    }
  }
}

void kvdrmon::updateToolTip()
{
  toolTip.setShutdown(autoShutdown ? stoptime : 0);
  toolTip.setRecording(running);
  toolTip.update(getNeededSpace());
}

void kvdrmon::toggleShutdown(bool checked)
{
  autoShutdown = checked;
  updateToolTip();
}

void kvdrmon::doShutdown()
{
      // shutdown?
  if (autoShutdown)
  {
    // no more processing
    shutdownMenu->setChecked(false);
    autoShutdown = false;

    // check for new recording inside wait time
    if (time(NULL) + prefs.shutdownWait > nextStart)
      return;

    // shutdown
    if (shutdownDialog)
      delete shutdownDialog;
    shutdownDialog = new shutdown(app);
    shutdownDialog->exec(prefs.shutdownWait);
  }
}

#include "kvdrmon.moc"
